define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/_FocusMixin",
	"dojo/Stateful",
	"dojo/store/Memory",
	"dojo/store/Observable",
	"dojo/Deferred",
	"dojo/dom",
	"dojo/query",
	"dojo/dom-geometry",
	"thornberry/serverRequest",
	"dojo/_base/window",
	"dojo/dom-style",
	"dojo/dom-construct",
	"dojo/_base/fx",
	"dojo/on",
	"dojo/_base/array",
	"dojo/date/locale",
	"dijit/MenuItem",
	"dijit/DropDownMenu",
	"dijit/form/Button",
	"dojo/NodeList-dom"
],function(declare,lang,_WidgetBase,_TemplatedMixin,_WidgetsInTemplateMixin,_FocusMixin,Stateful,Memory,Observable,Deferred,dom,query,domGeom,serverRequest,baseWindow,domStyle,domConstruct,fx,on,arrayUtils,locale,MenuItem) {
	/*
		Any page using this utility must include the element below in its HTML body.
		
			<iframe id="fileDownloadFrame" src="" style="display:none"></iframe>
		
		This widget also relies on thornberry/serverRequest, so pages must also include the form element required by that module.
		
		Use:
			The main methods used to interact with the report manager are
			1. addReport() - adds a report option button to the manager that users may click to generate the report
			2. show()/hide() - to be called by client code to cause the manager to be displayed and hidden when appropriate. The manager is automatically hidden when it loses focus and automatically shown when a report completes generation
			3. hideReport(), hideAllReports() - used to customize which report options are displayed to the user at any given time. Depending on the current page view or context, certain report options may not be appropriate. 
			
		If there is a button on the page separate from the report manager that is used to generate a report (such as a History button), the report should be added to the report manager using addReport(), and then, if desired, the returned report ID value from this method can be passed to hideReport() to prevent it from displaying in the reportManager itself. Then, the button's onClick event should pass the same report ID value to the report manager's generate() method to initiate report creation.
	*/
	
	var ReportItem = declare([_WidgetBase,_TemplatedMixin], {
		templateString: 
			"<div class='content-area-title reportManager' style='border-bottom:1px solid rgb(217,217,233)'>\
				<div class='content-area-title-text' data-dojo-attach-point='titleNode'>\
					<span>${reportName}</span>\
					<span data-dojo-attach-point='removeNode' style='float:right;font-weight:bold;font-size:0.75em;cursor: hand;cursor: pointer;'>X</span>\
				</div>\
				<div data-dojo-attach-point='statusNode' style='padding:4px'>Generating...</div>\
			</div>",
		
		reportName: "",
		promise: null,
		fileLinkId: "",
		error: "",
		completeTime: "",
		report: null,
		
		constructor: function(data) {
			var t = this;
			lang.mixin(t,data);
		},
		
		buildRendering: function() {
			var t = this;
			t.inherited(arguments);
			
			domStyle.set(t.titleNode,{
				"padding":"4px",
				"background-color":"rgb(140, 140, 140)",
				"color":"rgb(240,240,240)"
			});
			
			t.own(on(t.removeNode,"click",function() {
				t.remove();
			}));
		},
		
		openReport: function() {
			var t = this;
			if (t.fileLinkId !== "") {
				var link = ((t.fileLinkId !== undefined) && (t.fileLinkId[0] === '\\')) ? t.fileLinkId.substring(1, t.fileLinkId.length) : t.fileLinkId;
				dom.byId("fileDownloadFrame").src = link+"&prompt=true";				
			}
		},
		
		remove: function() {
			var t = this;
			if (t.promise !== null && !t.promise.isFulfilled()) {
				t.report.set("disabled",false);
				t.promise.cancel();
			}
		},
		
		displayReportError: function() {
			var t = this;
			t.statusNode.innerHTML = "There was a problem generating the report";
			domStyle.set(t.titleNode,"background-color","rgb(255, 128, 0)");
			
			if (t.report) {
				t.report.set("disabled",false);
			}
		},
		
		_setFileLinkIdAttr: function(fileLinkId) {
			var t = this;
			t.fileLinkId = fileLinkId;
			
			var statusTextNode = domConstruct.toDom("Click <a id='reportLinkNode'>here</a> to download <span style='float:right;color:rgb(110, 110, 110);font-size:0.75em;font-style:italic'>Completed at "+t.completeTime+"</span>");
			domConstruct.place(statusTextNode,t.statusNode,"only");
			
			domStyle.set(t.titleNode,"background-color","rgb(51, 204, 0)");
			
			t.own(on(dom.byId("reportLinkNode"),"click",function() {
				t.openReport();
			}));
		},
		
		_setPromiseAttr: function(promise) {
			var t = this;
			t.promise = promise;
			
			if (t.report) {
				t.report.set("disabled",true);
			}
			
			promise.then(function(fileLinkId) {
				if (fileLinkId !== "") {
					t.completeTime = locale.format(new Date(),{selector:"time",timePattern:"HH:mm:ss"});
					t.set("fileLinkId",fileLinkId);
					
					if (t.report) {
						t.report.set("disabled",false);
					}
				} else {
					t.displayReportError();
				}
			},function(error) {
				t.displayReportError();
			});
		},
		
		_serialize: function() {
			var t = this;
			return {
				reportName:t.reportName,
				fileLinkId:t.fileLinkId,
				completeTime:t.completeTime
			};
		}
	});
	ReportItem._deserialize = function(data) {
		return new ReportItem(data);
	};
	
	var Report = declare(Stateful, {
		id: "",
		name: "",
		reportClass: "",
		reportMethod: "Generate",
		type: "PDF",
		getOptions: null /*function*/,
		options: null,
		
		hidden: false,
		disabled: false,

		menuNode: null,
		
		constructor: function(data) {
			var t = this;
			lang.mixin(t,data);
			
			t.id = Report.id++;
		},
		
		generate: function() {
			var t = this;
			var reportItem = new ReportItem({
				reportName: t.name+" ("+t.type+")",
				report: t
			});
			
			var status = false;
			try {
				var options = t.getOptions(t.options);
				status = true;
			} catch (exception) {}
			
			if (status) {
				var reportPromise = serverRequest(t.reportClass,t.reportMethod,options,true);
				reportItem.set("promise",reportPromise);
			}
			
			return reportItem;
		},
		
		_disabledSetter: function(value) {
			var t = this;
			t.disabled = value;
			if (t.menuNode) {
				t.menuNode.set("disabled",value);
			}
		},
		
		_hiddenSetter: function(value) {
			var t = this;
			t.hidden = value;
			if (t.menuNode) {
				domStyle.set(t.menuNode.domNode,"display",(value ? "none" : ""));
			}
		}
	});
	
	Report.id = 1;
	
	var ReportManager = declare([_WidgetBase,_TemplatedMixin,_WidgetsInTemplateMixin,_FocusMixin], {
		templateString: 
			"<div>\
				<span data-dojo-attach-point='headerNode' class='smallCaps'>Report Manager</span>\
				<div data-dojo-type='dijit/DropDownMenu' data-dojo-attach-point='reportMenu' style='display:none'></div>\
				<div data-dojo-attach-point='reportContainerNode' style='margin-bottom:4px;overflow-y:auto;border:1px solid rgb(217,217,233)'></div>\
				<button data-dojo-attach-point='clearListButton' data-dojo-type='dijit/form/Button' style='float:right'>Clear</button>\
			</div>",
		
		reportStore: null, /* contains Report objects for all reports added to the manager in the current page */
		_itemStore: null, /* contains ReportItem objects for each initiated report generation that hasn't been cleared */
		
		width: 375,
		slideDuration: 600,
		isVisible: false, /* true if the report manager is currently displayed */
		
		postMixInProperties: function() {
			var t = this;
			t.inherited(arguments);
			
			t.reportStore = new Observable(new Memory());
			t._itemStore = new Observable(new Memory());
		},
		
		buildRendering: function() {
			var t = this;
			t.inherited(arguments);
			
			domStyle.set(t.domNode,{
				"z-index":5000,
				"display":"none",
				"position":"absolute",
				"top":"0px",
				"right":"-"+t.width+"px",
				"width":t.width+"px",
				"height":"100%",
				"background-color":"rgb(238,238,221)",
				"border":"1px solid rgb(217,217,233)",
				"padding":"8px",
				"box-sizing":"border-box"
			});
			
			domStyle.set(t.reportMenu.domNode,{
				"margin-top":"8px"
			});
			
			domStyle.set(t.reportContainerNode,{
				"margin-top":"8px",
				"background-color":"white",
			});
			
			arrayUtils.forEach(t.reportStore.data,function(report) {
				t._createReportNode(report);
			});
			
			baseWindow.body().appendChild(t.domNode);
			
			t.own(on(window,"resize",function() {
				t.resize();
			}));
		},
		
		postCreate: function() {
			var t = this;
			t.inherited(arguments);
			
			t.reportStore.query({}).observe(function(report,removedFrom,insertedInto) {
				if (insertedInto > -1) {
					t._createReportNode(report);
				}
				
				var visibleReports = t.reportStore.query({hidden:false});
				domStyle.set(t.reportMenu.domNode,"display",(visibleReports.length > 0 ? "" : "none"));
				
				t.resize();
			});
			
			t._itemStore.query({}).observe(function(item,removedFrom,insertedInto) {
				if (insertedInto > -1) {
					domConstruct.place(item.domNode,t.reportContainerNode,"first");
					
					t.own(on(item.removeNode,"click",function() {
						t._itemStore.remove(item.id);
					}));
				}
				if (removedFrom > -1) {
					item.destroy();
					//t._saveInSession();
				}
			});
			
			t.own(on(t.clearListButton,"click",function() {
				for (var i = t._itemStore.data.length; i--; i >= 0) {
					t._itemStore.data[i].remove();
					t._itemStore.remove(t._itemStore.data[i].id);
				}
			}));
			
			//t._retrieveFromSession();
		},
		
		addReport: function(reportName,reportClass,reportMethod,/*function*/getOptions,options,type) {
			/*
				Description: adds a button to the manager that when clicked generates a report
				Inputs:
					reportName: the display name of the report
					reportClass: the class responsible for generating the report
					reportMethod: the method to be called on reportClass to generate the report. This method must return a file link ID to the generated file
					getOptions: a function that gathers and returns the options to be passed to the report generation method
					options: optional; this value gets passed to getOptions() when it is called
					type: optional; this value determines how the generated file is displayed. The default is "PDF". The other options "Excel". See _createReportNode()
				Output: an ID value that can be used later to reference this report, for example, to hide the report when a UI context change makes it not applicable. See hideReport()
			*/
			var t = this;
			type = type || "PDF";
			
			var report = new Report({
				name: reportName,
				reportClass: reportClass,
				reportMethod: reportMethod,
				getOptions: getOptions,
				options: options,
				type: type
			});
			t.reportStore.put(report);
			return report.id;
		},
		
		hideReport: function(reportId,hidden) {
			var t = this;
			if (typeof hidden === "undefined") {
				hidden = true;
			}
			var report = t.reportStore.get(reportId);
			report.set("hidden",hidden);
			var visibleReports = t.reportStore.query({hidden:false});
			domStyle.set(t.reportMenu.domNode,"display",(visibleReports.length > 0 ? "" : "none"));
			t.resize();
			return t;
		},
		
		hideAllReports: function(hidden) {
			var t = this;
			if (typeof hidden === "undefined") {
				hidden = true;
			}
			arrayUtils.forEach(t.reportStore.data,function(report) {
				report.set("hidden",hidden);
			});
			var visibleReports = t.reportStore.query({hidden:false});
			domStyle.set(t.reportMenu.domNode,"display",(visibleReports.length > 0 ? "" : "none"));
			t.resize();
			return t;
		},
		
		generate: function(/*report object or id*/report) {
			var t = this;
			
			if (typeof report !== "object") {
				report = t.reportStore.get(report);
			}
			
			t.show();
			
			var reportItem = report.generate();
			if (reportItem.promise !== null) {
				t._itemStore.put(reportItem);
				
				reportItem.promise.then(function() {
					t.show();
					//t._saveInSession();
				},function() {
					t.show();
				});
			}
		},
		
		show: function() {
			var t = this;
			if (!t.isVisible) {
				t.isVisible = true;
				domStyle.set(t.domNode,"display","");
				t.resize();
				fx.animateProperty({
					node: t.domNode,
					properties: {
						right: 0,
					},
					onEnd: function() {
						t._focusManager.focus(t.domNode);
						t.isVisible = true;
					},
					duration: t.slideDuration
				}).play();
			}
		},
		
		hide: function() {
			var t = this;
			t.isVisible = false;
			fx.animateProperty({
				node: t.domNode,
				properties: {
					right:(-t.width)
				},
				onEnd: function() {
					if (t.isVisible === false) {
						domStyle.set(t.domNode,"display","none");
					}
				},
				duration: t.slideDuration
			}).play();
		},
		
		_onBlur: function() {
			var t = this;
			t.hide();
		},
		
		_retrieveFromSession: function() {
			var t = this;
			try {
				if (typeof top.reportManagerData !== "undefined") {
					arrayUtils.forEach(top.reportManagerData,function(itemData) {
						t._itemStore.put(ReportItem._deserialize(itemData));
					});
				}
			} catch (e) {}
		},
		
		_saveInSession: function() {
			var t = this;
			try {
				var serializedArray = [];
				arrayUtils.forEach(t._itemStore.data,function(item) {
					if (item.completeTime !== "") {
						serializedArray.push(item._serialize());
					}
				});
				top.reportManagerData = serializedArray;
			} catch (e) {}
		},
		
		_createReportNode: function(report) {
			var t = this;
			
			var action = "";
			switch(report.type) {
				case "PDF":
					action = "Print";
					break;
				case "Excel":
					action = "Export";
					break;
				default:
					action = "Print";
			}
				
			var reportNode = new MenuItem({
				label: action+" "+report.name,
				onClick: function() {
					t.generate(report);
					t.reportMenu.set("selected",null);
				}
			});
			
			domStyle.set(reportNode.domNode,"white-space","normal");
			t.reportMenu.addChild(reportNode);
			report.menuNode = reportNode;
		},
		
		resize: function() {
			var t = this;
			var totalHeight = domGeom.getContentBox(t.domNode).h;
			var headerBox = domGeom.getMarginBox(t.headerNode);
			var menuBox = domGeom.getMarginBox(t.reportMenu.domNode);
			var buttonBox = domGeom.getMarginBox(t.clearListButton.domNode);
			var height = totalHeight - headerBox.h - menuBox.h - buttonBox.h - 8;
			if (height < 200) {
				height = 200;
			}
			domStyle.set(t.reportContainerNode,"height",height+"px");
		}
	});
	
	return ReportManager;
});