/*	
	Author: *Unknown*
	Created: *Unknown*
	Purpose: Report Manager II
	History:
    2022-09-22 DJA ND-12086 Added calls to freeze/unfreeze the report manager during button clicks.  This prevents
							the report manager from closing prematurely.  Also, added call to focus report manager 
							after a button click to re-enable the blur event when clicking outside of the report manager.
    2022-09-09 JCF Added an id to queryNode as queryNode, added dombyId click function to focus the sql window
	2018-10-03 WRA ND-7611 Allowing primary reports to display full title instead of "Current View"
*/
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
    "dijit/_FocusMixin",
    "dijit/registry",
	"dojo/Stateful",
	"dojo/store/Memory",
	"dojo/store/Observable",
	"dojo/Deferred",
	"dojo/dom",
    "dojo/query",
    "dojo/when",
	"dojo/dom-geometry",
    "thornberry/serverRequest",
    "thornberry/report/Report",
    "thornberry/UserSettings",
	"dojo/_base/window",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/dom-attr",
	"dojo/_base/fx",
	"dojo/on",
	"dojo/_base/array",
	"dojo/date/locale",
    "dijit/form/Button",
    "dijit/form/CheckBox",
    "dojox/form/CheckedMultiSelect",
    "dijit/form/Select",
    "dijit/form/MultiSelect",
    "dijit/Tooltip",
    "dojo/NodeList-dom"
],function(declare,lang,_WidgetBase,_TemplatedMixin,_WidgetsInTemplateMixin,_FocusMixin,registry,Stateful,Memory,Observable,Deferred,dom,query,when,domGeom,serverRequest,Report,UserSettings,baseWindow,domStyle,domClass,domConstruct,domAttr,fx,on,arrayUtils,locale,Button,CheckBox,CheckedMultiSelect,Select,MultiSelect,Tooltip) {

    var SLIDEDURATION = 600;
    var WIDTH = 400;

    var GeneratedItem = declare([_WidgetBase,_TemplatedMixin],{
        templateString:
            "<div class='generated-item'>" +
                "<div data-dojo-attach-point='headerNode' class='header header-incomplete'><span data-dojo-attach-point='reportNameNode'>${report.displayName} (${presentationType})</span>" +
                    "<div data-dojo-attach-point='removeNode' title='Remove' class='icon stop' style='float:right'></div>" +
                "</div>" +
                "<div data-dojo-attach-point='statusNode' class='status description'>Generating...</div>" +
                "<div data-dojo-attach-point='buttonNode' class='statusButtons description' style='display:none'>" +
                    "<div data-dojo-attach-point='downloadButton'>Open</div>" +
                    "<div data-dojo-attach-point='detailButton'>Details</div>" +
                "</div>" +
                "<div data-dojo-attach-point='infoNode' class='description status' style='display:none'>" +
                    "<div data-dojo-attach-point='downloadButton2' style='cursor:hand;cursor:pointer;display:inline-block;'><div title='Download' class='icon download' style='margin-right:7px'></div><span>Open</span></div>" +
                    "<span class='time-display'>Completed at <span data-dojo-attach-point='timeNode'></span></span>" +
                    "<div data-dojo-attach-point='detailNode'></div>" +
                    "<div style='text-align:center'><div data-dojo-attach-point='contractButton' class='icon wide-arrow'></div></div>" +
                "</div>" +
                "<div data-dojo-attach-point='errorNode' class='status description' style='display:none'>There was an error generating the report</div>" +
            "</div>",

        report: null,
        manager: null,
        presentationType: "",
        options: null, // object with values for each report option
        status: "incomplete", // "incomplete", "not-downloaded", "downloaded", "error"
        context: null,

        _promise: null,

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);
            domConstruct.place(t.domNode,t.manager.generatedItemContainer,"first");
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);
            t.generateReport();

            t.own(on(t.removeNode,"click",function() {
                t.remove();
            }));
        },

        remove: function() {
            this.destroy();
        },

        generateReport: function() {
            var t = this;
            t.context = t.manager.getContext(t.presentationType,t.report);

            if (t.context.options && typeof t.context.options.presetName !== "undefined" && t.context.options.presetName !== "") {
                t.reportNameNode.innerHTML = escapeText(t.context.options.presetName+" ("+t.presentationType+")");
            }

            if (t.context) {
                lang.mixin(t.context.options,t.options);

                if (!t.report.childReport) {
                    var promise = serverRequest(t.report.className,"GenerateFile",t.context,true).then(lang.hitch(t,"onComplete"),lang.hitch(t,"onError"));
                } else {
                    var promise = serverRequest(t.manager.report.className,"GenerateChildReport",t.context,true).then(lang.hitch(t,"onComplete"),lang.hitch(t,"onError"));
                }
                t._promise = promise;
            } else {
                t.defer("remove");
            }
        },

        onComplete: function(fileLinkId) {
            var t = this;

            if (fileLinkId !== "") {
                t.set("status","not-downloaded");

                t.timeNode.innerHTML = locale.format(new Date(),{selector:"time",timePattern:"HH:mm:ss"});

                t.own(
                    on(t.detailButton,"click",function() {t.toggleDetails();}),
                    on(t.contractButton,"click",function() {t.toggleDetails();}),
                    on(t.downloadButton,"click",function() {
                        t.set("status","downloaded");
                        dom.byId("fileDownloadFrame").src = fileLinkId+"&prompt=true";
                    }),
                    on(t.downloadButton2,"click",function() {
                        t.set("status","downloaded");
                        dom.byId("fileDownloadFrame").src = fileLinkId+"&prompt=true";
                    })
                );

                t.setDetails();
            } else {
                t.onError();
            }
        },

        toggleDetails: function(hide) {
            var t = this;
            if (t.status !== "incomplete" && t.status !== "error") {
                if (typeof hide === "undefined") {
                    hide = (domStyle.get(t.infoNode,"display") === "none" ? false : true);
                }

                if (!hide) {
                    domStyle.set(t.infoNode,"display","");
                    domStyle.set(t.buttonNode,"display","none");
                } else {
                    domStyle.set(t.infoNode,"display","none");
                    domStyle.set(t.buttonNode,"display","");
                }
            }
        },

        onError: function(message) {
            var t = this;
            if (message === "No results found to print") {
                if (typeof actionShowMessage !== "undefined" && actionShowMessage) {
                    actionShowMessage(message);
                } else {
                    alert(message);
                }
                t.remove();
            } else {
                t.set("status","error");
            }
        },

        setDetails: function() {
            var t = this;

            // Add options
            var labelNode = "";
            var optionCount = 0;
            for (optionName in t.context.options) {
                optionCount++;
                break;
            }
            if (optionCount > 0) {
                var first = true;
                var listNode = null;
                var createLabel = function() {
                    labelNode = domConstruct.create("div",{innerHTML:"Options"},t.detailNode);
                    domClass.add(labelNode,"top-spacer-small");
                    domClass.add(labelNode,"title");
                    listNode = domConstruct.create("ul",{},t.detailNode);
                };
                for (optionName in t.context.options) {
                    var optionDefinition = t.report.getOption(optionName);
                    var optionValue = "";
                    if (optionDefinition) {
                        if (optionDefinition.type === "CheckBox") {
                            optionValue = (t.options[optionName] ? "Yes" : "No");
                        } else {
                            arrayUtils.some(optionDefinition.options,function(option) {
                                if (option.id === t.options[optionName]) {
                                    optionValue = option.name;
                                    return true;
                                }
                                return false;
                            });
                        }
                        if (first) {createLabel(); first = false;}
                        domConstruct.create("li",{innerHTML:optionDefinition.displayName+" = "+optionValue},listNode);
                    } else if (optionName === "selectedIds") {
                        // Rows selected in the grid
                        if (first) {createLabel(); first = false;}
                        domConstruct.create("li",{innerHTML:"Rows Selected = "+t.context.options[optionName].length},listNode);
                    } else if (optionName === "presetName") {
                        if (first) {createLabel(); first = false;}
                        domConstruct.create("li",{innerHTML:"Preset Name = "+t.context.options[optionName]},listNode);
                    } else if (optionName === "summarizeBy") {
                        if (first) {createLabel(); first = false;}
                        if (typeof t.context.options.rowValue !== "undefined") {
                            var pieces = t.context.options[optionName].toString().split(",");
                            domConstruct.create("li",{innerHTML:pieces[0]+" = "+t.context.options.rowValue},listNode);
                            domConstruct.create("li",{innerHTML:pieces[1]+" = "+t.context.options.columnValue},listNode);
                        } else {
                            domConstruct.create("li",{innerHTML:"Summarize By = "+t.context.options[optionName]},listNode);
                        }
                    }
                }
            }

            // Add filters
            if (t.context.filters.length > 0) {
                var first = true;
                var listNode = null;
                var createLabel = function() {
                    labelNode = domConstruct.create("div",{innerHTML:"Filters"},t.detailNode);
                    domClass.add(labelNode,"top-spacer-small");
                    domClass.add(labelNode,"title");
                    listNode = domConstruct.create("ul",{},t.detailNode);
                };
                arrayUtils.forEach(t.context.filters,function(filter) {
                    if (typeof filter.displayValue !== "undefined" && filter.displayValue !== "") {
                        if (first) {createLabel(); first = false;}
                        domConstruct.create("li",{innerHTML:filter.displayName+" = "+filter.displayValue},listNode);
                    }
                });
            }

            // Add summary
            if (t.presentationType === "PDF" && t.context.summarizeBy && t.context.summarizeBy !== "") {
                var columnDefinition = t.report.getColumn(t.context.summarizeBy);
                domConstruct.create("li",{innerHTML:"Summarize by = "+(columnDefinition ? columnDefinition.displayName : t.context.summarizeBy)},listNode);
            }
        },

        _setStatusAttr: function(status) {
            var t = this;
            t.status = status;

            domClass.add(t.headerNode,"header-"+status);
            if (status === "downloaded") {
                domClass.remove(t.headerNode,"header-not-downloaded");
            } else if (status === "not-downloaded") {
                domClass.remove(t.headerNode,"header-incomplete");
                domStyle.set(t.statusNode,"display","none");
                domStyle.set(t.buttonNode,"display","");
            } else if (status === "error") {
                domClass.remove(t.headerNode,"header-incomplete");
                domStyle.set(t.statusNode,"display","none");
                domStyle.set(t.errorNode,"display","");
            }
            if (status !== "incomplete") {
                t.manager.show();
            }
        }
    });

    var _OptionMixin = declare([_WidgetBase,_TemplatedMixin],{
        templateString: "",
        optionDefinition: null,
        reportControl: null,
        widget: null,

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            if (t.optionDefinition.help !== "") {
                t.defer(function() {
                    new Tooltip({
                        connectId: [t.domNode.id],
                        label: escapeText(t.optionDefinition.help)
                    });
                });
            }
        },

        postCreate: function() {
            var t = this;
            t.widget._onChangeActive = false;

            var hasSaved = false;
            if (t.optionDefinition.retainValue && t.reportControl.manager.userSettings) {
                if (t.reportControl.manager.userSettings.has("ReportOption",t.optionDefinition.name)) {
                    t.deserialize(t.reportControl.manager.userSettings.get("ReportOption",t.optionDefinition.name));
                    hasSaved = true;
                }
                t.own(on(t.widget,"change",function() {
                    t.reportControl.manager.userSettings.set("ReportOption",t.optionDefinition.name,t.serialize(),true);
                }));
            }
            if (!hasSaved) {
                t.deserialize(t.optionDefinition.defaultValue);
            }

            t.widget._onChangeActive = true;

            t.own(on(t.widget,"change",function() {
                t.reportControl.options[t.optionDefinition.name] = t.getValue();
            }));
            t.reportControl.options[t.optionDefinition.name] = t.getValue();
        },

        getValue: function() {
            return this.widget.get("value");
        },

        serialize: function() {
            return this.widget.get("value");
        },

        deserialize: function(value) {
            this.widget.set("value",value);
        }
    });

    var CheckBoxOption = declare([_OptionMixin],{
        templateString:
            "<div class='option-node'>" +
                "<input data-dojo-attach-point='widgetNode'></input>" +
                "<div data-dojo-attach-point='labelNode' class='option-label'>${optionDefinition.displayName}</div>" +
            "</div>",

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);
            t.widget = new CheckBox({},t.widgetNode);
            t.widget.startup();

            t.own(on(t.labelNode,"click",function() {
                t.widget.set("checked",!t.widget.get("checked"));
            }));
        },

        getValue: function() {
            return this.widget.get("checked");
        },

        serialize: function() {
            return this.widget.get("checked");
        },

        deserialize: function(value) {
            this.widget.set("checked",value);
        }
    });

    var RadioOption = declare([_OptionMixin],{
        templateString:
            "<div class='option-node radioSelect'>" +
                "<div data-dojo-attach-point='labelNode' class='option-label'>${optionDefinition.displayName}</div>" +
                "<input data-dojo-attach-point='widgetNode'></input>" +
            "</div>",

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.widget = new CheckedMultiSelect({
                multiple: false
            },t.widgetNode);
            arrayUtils.forEach(t.optionDefinition.options,function(optionObject) {
                var option = domConstruct.create("option",{label:escapeText(optionObject.name),value:optionObject.id});
                t.widget.addOption(option);
            });
        }
    });

    var SelectOption = declare([_OptionMixin],{
        templateString:
            "<div class='option-node'>" +
                "<div data-dojo-attach-point='labelNode' class='option-label'>${optionDefinition.displayName}</div>" +
                "<input data-dojo-attach-point='widgetNode'></input>" +
            "</div>",

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.widget = new Select({},t.widgetNode);

            var optionStore = new Memory({
                data: t.optionDefinition.options,
                getLabel: t.getName
            });
            t.widget.setStore(optionStore);
            t.widget.startup();
        },

        getName: function(item) {
            return item.name;
        }
    });

    var MultiSelectOption = declare([_OptionMixin],{
        templateString:
            "<div class='option-node radioSelect'>" +
                "<div data-dojo-attach-point='labelNode' class='option-label' style='vertical-align:top'>${optionDefinition.displayName}</div>" +
                "<input data-dojo-attach-point='widgetNode'></input>" +
            "</div>",

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);
            t.widget = new MultiSelect({
                size: 6
            },t.widgetNode);

            arrayUtils.forEach(t.optionDefinition.options,function(optionObject) {
                var option = domConstruct.create("option",{label:escapeText(optionObject.name),value:optionObject.id});
                t.widget.domNode.appendChild(option);
            });
        },

        serialize: function() {
            return this.widget.get("value").join(",");
        },

        deserialize: function(value) {
            this.widget.set("value",value.toString().split(","));
        }
    });


    var ReportControl = declare([_WidgetBase,_TemplatedMixin,_WidgetsInTemplateMixin,_FocusMixin],{
        templateString:
            "<div class='report-display'>" +
                '<div data-dojo-attach-point="backButton" title="Back" class="icon back-icon" style=""></div>' +
                "<div data-dojo-attach-point='titleNode' class='workspace-title'>${displayName}</div>" +
                "<div data-dojo-attach-point='descriptionNode' class='description'>${report.description}</div>" +
                "<div data-dojo-attach-point='optionDiv'>" +
                    "<div data-dojo-attach-point='optionArea' style='overflow-y:auto'>" +
                        "<div data-dojo-attach-point='optionTitle' class='title top-spacer'>Options</div>" +
                    "</div>" +
                    "<div class='button-container top-spacer' data-dojo-attach-point='exportOptions'></div>" +
                "</div>" +
                "<div data-dojo-attach-point='queryDiv' style='display:none'>" +
                    "<pre data-dojo-attach-point='queryNode' class='query-text' style='overflow-y:auto'>${queryText}</pre>" +
                    "<div class='button-container top-spacer' data-dojo-attach-point='copyButtonDiv'></div>" +
                "</div>" +
            "</div>",

        report: null,
        manager: null,
        options: null,
        displayName: "",
        queryDisplayed: false,
        queryText: "",
		optionWidgets: null,

        resize: function() {
            var t = this;
            t.inherited(arguments);

            var totalHeight = domGeom.getContentBox(t.domNode.parentNode).h;
            var titleHeight = domGeom.getContentBox(t.titleNode).h;
            var descHeight = domGeom.getContentBox(t.descriptionNode).h;

            if (domStyle.get(t.optionDiv,"display") !== "none") {
                var buttonHeight = domGeom.getContentBox(t.exportOptions).h;
                var optionHeight = totalHeight - titleHeight - descHeight - buttonHeight - 60;
                optionHeight = (optionHeight < 0 ? 0 : optionHeight);
                domStyle.set(t.optionArea,"height",optionHeight+"px");
            }
            if (domStyle.get(t.queryDiv,"display") !== "none") {
                var copyButtonHeight = domGeom.getContentBox(t.copyButtonDiv).h;
                var queryHeight = totalHeight - titleHeight - descHeight - copyButtonHeight - 72;
                queryHeight = (queryHeight < 0 ? 0 : queryHeight);
                domStyle.set(t.queryNode,"height",queryHeight+"px");
            }
		},

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

			t.optionWidgets = [];
            t.options = {};
            t.queryText = t.report.queryText.toString().replace(/\t/g,"  ");
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            if (t.report.optionDefinitions.length === 0) {
                domStyle.set(t.optionTitle,"display","none");
            } else {
                arrayUtils.forEach(t.report.optionDefinitions,function(optionDefinition) {
                    if (optionDefinition.isDisplayed) {
                        var optionWidget = "";
                        var TypeClass = CheckBoxOption;
                        if (optionDefinition.type === "Radio") {
                            TypeClass = RadioOption;
                        } else if (optionDefinition.type === "Select") {
                            TypeClass = SelectOption;
                        } else if (optionDefinition.type === "MultiSelect") {
                            TypeClass = MultiSelectOption;
                        }
                        optionWidget = new TypeClass({
                            optionDefinition: optionDefinition,
                            reportControl: t
                        });
                        if (optionWidget !== "") {
                            t.optionArea.appendChild(optionWidget.domNode);
                        }
						t.optionWidgets.push(optionWidget);
                        //t.options[optionDefinition.name] = optionDefinition.defaultValue;
                    }
                });
            }

            arrayUtils.forEach(t.report.presentationTypes.toString().split(","),function(presentationType) {
                if (presentationType !== "Interactive") {
                    new Button({
                        label: (presentationType === "PDF" ? "Print" : "Export "+presentationType),
                        onClick: function() {
							t.manager.freeze(true);
                            t.createGeneratedItem(presentationType);
							setTimeout(function(){t.manager.freeze(false);t.manager.focusSelf();},100);
                        }
                    }).placeAt(t.exportOptions);
                }
            });

            if (t.report.queryText !== "") {
                new Button({
                    label: "View Query",
                    onClick: function() {
                        t.manager.freeze(true);
                        t.showQuery(true);
                        setTimeout(function(){t.manager.freeze(false);t.manager.focusSelf();},100);
                    }
                }).placeAt(t.exportOptions);
                new Button({
                    label: "Copy to Clipboard",
                    onClick: function() {
                        t.manager.freeze(true);
                        try{
							window.clipboardData.setData("Text",t.report.queryText);
						} catch(e) { console && console.log(e); }
                        setTimeout(function(){t.manager.freeze(false);t.manager.focusSelf();},100);
                    }
                }).placeAt(t.copyButtonDiv);
            }

            t.manager.onBuildControlRendering(t);
        },

        showQuery: function(show) {
            var t = this;
            domStyle.set(t.optionDiv,"display",(show ? "none" : ""));
            domStyle.set(t.queryDiv,"display",(show ? "" : "none"));
            t.resize();
            t.queryDisplayed = show;
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            t.own(on(t.backButton,"click",function() {
                if (t.queryDisplayed) {
                    t.showQuery(false);
                } else {
                    t.manager.showReportView();
                }
            }));
        },

        createGeneratedItem: function(presentationType) {
            var t = this;
            var generatedItem = new GeneratedItem({
                report: t.report,
                manager: t.manager,
                options: t.options,
                presentationType: presentationType
            });
            if (generatedItem._promise) {
                t.manager.showReportView();
            }
			return generatedItem;
        }
    });

    var ReportItem = declare([_WidgetBase,_TemplatedMixin],{
        templateString:
            '<div class="report-item">' +
                '<div class="title" data-dojo-attach-point="titleNode">${displayName}</div>' +
                '<div class="description">${report.shortDescription}</div>' +
            '</div>',

        report: null,

        manager: null,

        currentView: false,
        displayName: "",

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);
            t.displayName = (t.currentView ? 'Current View' : t.report.displayName);
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            if (t.currentView) {
                domClass.add(t.titleNode, 'emphasize');
            }

            t.own(on(t.domNode,"click",function() {
                t.manager.showReport(t.report,t.displayName);
            }));
        }
    });

    var ReportManager = declare([_WidgetBase,_TemplatedMixin,_WidgetsInTemplateMixin,_FocusMixin], {
		templateString:
            "<div class='report-manager'>" +
                "<div data-dojo-attach-point='viewStack' style='height:100%;width:100%' data-dojo-type='dijit/layout/StackContainer'>" +
                    "<div data-dojo-attach-point='reportView' data-dojo-type='dijit/layout/ContentPane'>" +
                        "<div data-dojo-attach-point='reportHeader' class='workspace-title'>Reports</div>" +
                        "<div data-dojo-attach-point='reportItemContainer' class='item-area report-item-container'></div>" +
                        "<div data-dojo-attach-point='generatedHeader' class='workspace-title top-spacer'>Generated</div>" +
                        "<div data-dojo-attach-point='generatedItemContainer' class='item-area'></div>" +
                        "<button class='right' data-dojo-attach-point='clearButton' data-dojo-type='dijit/form/Button'>Clear</button>" +
                    "</div>" +
                    "<div data-dojo-attach-point='optionView' data-dojo-type='dijit/layout/ContentPane'>" +

                    "</div>" +
                    "<div data-dojo-attach-point='queryView' data-dojo-type='dijit/layout/ContentPane'>" +
                    "</div>" +
                "</div>" +
            "</div>",

        report: null, // instance of thornberry/report/Report

        _isVisible: false, // true if the report manager is currently displayed,

        getContext: null, // a method supplied by the page to get the report context. It is passed the report definition and presentation type. It returns any falsy value if there's a problem, and must provide adequate messaging to the user

        reportStore: null,

        userSettings: null,

        frozen: false, // if true, then when the manager is shown, it will not be hidden even if it loses focus

        currentViewTitle: true, // if false, use the report title instead of "Current View"

        onBuildControlRendering: function(reportControl) {},

        postMixInProperties: function() {
			var t = this;
			t.inherited(arguments);

            t.reportStore = new Memory({
                idProperty: "className"
            });

            try {
                if (t.userSettings === null) {
					if (typeof generalUserSettings !== "undefined") {
						t.userSettings = generalUserSettings;
					} else {
						if (typeof userId === "undefined") {
							var id = serverRequest("UI.Utility","GetUserId",{});
						} else {
							var id = userId;
						}
						when(id,function(value) {
							t.userSettings = new UserSettings({
								userId: value,
								category: "_GENERAL"
							});
							t.userSettings.load();
						});
					}
                }
            } catch (e) {console.error(e);}
        },

        buildRendering: function() {
			var t = this;
			t.inherited(arguments);

            domStyle.set(t.domNode,"right","-"+WIDTH+"px");
            domStyle.set(t.domNode,"width",WIDTH+"px");

            baseWindow.body().appendChild(t.domNode);
        },
		
		_setReportAttr: function(report) {
			var t = this;
			if (report) {
				t.reportStore.setData([]);
				
				arrayUtils.forEach(registry.findWidgets(t.reportItemContainer),function(widget) {
					widget.destroy();
				});
				domConstruct.empty(t.reportItemContainer);
				
				t.report = report;
				if (t.report.hasFilePresentation()) {
					var reportItem = new ReportItem({
						report: t.report,
						manager: t,
						currentView: true && t.currentViewTitle
					});
					t.reportItemContainer.appendChild(reportItem.domNode);
					t.reportStore.put(t.report);
				}

				arrayUtils.forEach(t.report.childReportDefinitions,function(reportDefinition) {
					var report = new Report(reportDefinition);
					if (report.hasFilePresentation()) {
						var reportItem = new ReportItem({
							report: report,
							manager: t
						});
						t.reportItemContainer.appendChild(reportItem.domNode);
						t.reportStore.put(report);
					}
				});
			}  else {
                domStyle.set(t.reportItemContainer,"display","none");
                domStyle.set(t.reportHeader,"display","none");
				domClass.remove(t.generatedHeader,"top-spacer");
            }
			
			if (t._started) {
				t.resize();
			}
		},
			
        toggleReportItem: function(reportClass,show) {
            var t = this;
            var reportItems = registry.findWidgets(t.reportItemContainer);
            arrayUtils.some(reportItems,function(reportItem) {
                var found = false;
                if (reportItem.report.className === reportClass) {
                    found = true;
                    domStyle.set(reportItem.domNode,"display",(show ? "" : "none"));
                }
                return found;
            });


            var visibleReports = arrayUtils.some(reportItems,function(reportItem) {
                return domStyle.get(reportItem.domNode,"display") !== "none";
            });
            if (visibleReports && domStyle.get(t.reportItemContainer,"display") === "none") {
                domStyle.set(t.reportItemContainer,"display","");
                domStyle.set(t.reportHeader,"display","");
            } else if (!visibleReports && domStyle.get(t.reportItemContainer,"display") !== "none") {
                domStyle.set(t.reportItemContainer,"display","none");
                domStyle.set(t.reportHeader,"display","none");
            }
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            t.own(on(t.clearButton,"click",function() {
                t.clear();
            }));

            t.own(on(window,"resize",function() {
				t.resize();
			}));
        },

        showReport: function(report,displayName) {
            var t = this;
            var reportControl = new ReportControl({
                report: report,
                manager: t,
                displayName: displayName
            });

            domConstruct.empty(t.optionView.domNode);
            t.optionView.domNode.appendChild(reportControl.domNode);
            t.viewStack.selectChild(t.optionView);
        },

        showReportView: function() {
            var t = this;
            t.viewStack.selectChild(t.reportView);
            query(".generated-item",t.generatedItemContainer).forEach(function(node) {
                registry.byId(node.id).toggleDetails(true);
            });
            t.resize();
        },
		
		focusSelf: function() {
			var t = this;
            t._focusManager.focus(t.domNode);
		},

        show: function() {
			var t = this;
			if (!t._isVisible) {
                if (t.userSettings) {
                    t.userSettings.load();
                }

                t._isVisible = true;
				domStyle.set(t.domNode,"display","block");
                t.showReportView();
				fx.animateProperty({
					node: t.domNode,
					properties: {
						right: 0,
					},
					onEnd: function() {
                        t._isVisible = true;
                        t.focusSelf();
					},
					duration: SLIDEDURATION
				}).play();
			}
		},

		hide: function() {
            var t = this;
			t._isVisible = false;
			fx.animateProperty({
				node: t.domNode,
				properties: {
					right:(-WIDTH)
				},
				onEnd: function() {
					if (t._isVisible === false) {
						domStyle.set(t.domNode,"display","none");
					}
				},
				duration: SLIDEDURATION
			}).play();
        },

        clear: function() {
            var t = this;
            query(".generated-item",t.generatedItemContainer).forEach(function(node) {
                registry.byId(node.id).remove();
            });
        },

		_onBlur: function() {
            var t = this;
            if (!t.frozen) {
                this.hide();
            }
        },

        resize: function() {
            var t = this;
            t.inherited(arguments);
            t.viewStack.resize();
            var totalHeight = domGeom.getContentBox(t.domNode).h;
            var reportAreaHeight = domGeom.getContentBox(t.reportItemContainer).h;
			var height = totalHeight - reportAreaHeight - (t.report ? 140 : 90);
			if (height < 200) {
				height = 200;
			}
            domStyle.set(t.generatedItemContainer,"height",height+"px");
        },

        addExternalReport: function(report,presentationType) {
            var t = this;
            var generatedItem = new GeneratedItem({
                manager: t,
                report: report,
                presentationType: presentationType,
                options: t.getDefaultOptions(report)
            });
            t.show();
            return generatedItem;
        },

        generateReport: function(reportClass,presentationType,options) {
            var t = this;
            options = options || {};
            presentationType = presentationType || "PDF";
			report = (lang.isString(reportClass) ? t.reportStore.get(reportClass) : reportClass); //
            var generatedItem = new GeneratedItem({
                report: report,
                manager: t,
                options: options,
                presentationType: presentationType
            });
            t.show();
        },

        freeze: function(frozen) {
            var t = this;
            if (typeof frozen === "undefined") {
                frozen = true;
            }
            t.frozen = !!frozen;
        },

        getDefaultOptions: function(report) {
            // Description:
            //      Utility function for getting the default options values applicable to a report for a certain user. Useful when getContext is called after a
            //      report generation is triggered by addExternalReport()
            var t = this;
            var options = {};
            arrayUtils.forEach(report.optionDefinitions,function(optionDefinition) {
                if (t.userSettings.has("ReportOption",optionDefinition.name)) {
                    options[optionDefinition.name] = t.userSettings.get("ReportOption",optionDefinition.name);
                }
            });
            return options;
        }
	});
	
	ReportManager.ReportControl = ReportControl;
	ReportManager.GeneratedItem = GeneratedItem;

	return ReportManager;
});
