/*
    Author: Will Amesbury
    Created: 2022-09-22
    History:

    Notes: Uses much of the Billing Management infrastructure
*/
define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/json",
	"dojo/dom-construct",
	"thornberry/app/Component/_Component",
	"thornberry/app/Component/_ViewMixin",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./Templates/Dashboard.html",
	"thornberry/app/service/BillingManagement",
	"thornberry/app/utils",
	"thornberry/app/Component/ScoreCard",
	"dojo/currency",
	"thornberry/app/utils"
],function(
	declare,
	lang,
	arrayUtils,
	json,
	domConstruct,
	_Component,
	_ViewMixin,
	_TemplatedMixin,
	_WidgetsInTemplateMixin,
	template,
	service,
	utils,
	ScoreCard,
	currency
) {
	var canViewFinancialInfo = false;
	utils.checkPermission("BillingViewFinancialInfo",false).then(function(allowed){canViewFinancialInfo=!!allowed});

	return declare([_Component,_ViewMixin,_TemplatedMixin,_WidgetsInTemplateMixin],{
		title: "Dashboard",

		templateString: template,

		data: null,

		getPageProps: function() {
			var t = this;
			var actions = [
				{ label: "Refresh", onClick: lang.hitch(t,"refreshData") }
			]
			return lang.mixin(t.inherited(arguments),{
				content: t.domNode,
				actions: actions,
				helpButtonShown: true
			});
		},

		buildRendering: function() {
			var t = this;
			t.inherited(arguments);

			var getNetDisplay = function(key,value) {
				if (key == "net") {
					value = (value === "" ? "$ - " : currency.format(value,{currency:"USD"}));
				}
				return value;
			};

			var countNetTemplate = 
				"<div class='flex-field-container'>"+
					"<div class='field-name field-name-grand'>${props.item.name}</div>"+
					"<div class='field-value field-value-medium-small' style='text-align:right'>${props.item.count}</div>"+
					"<div class='field-value field-value-medium' style='text-align:right'>${props.item.net}</div>"+
				"</div>";

			/*var countTemplate = 
				"<div class='flex-field-container'>"+
					"<div class='field-name field-name-grand'>${props.item.name}</div>"+
					"<div class='field-value field-value-medium-small' style='text-align:right'>${props.item.count}</div>"+
				"</div>";*/

			t.unbilledClaimsCard = new ScoreCard({
				title: "Unbilled Demand Claims",
				props: {
					rowTemplate: countNetTemplate,
					getDisplayValue: getNetDisplay
				}
			});
			t.cardContainerLeft.appendChild(t.unbilledClaimsCard.domNode);
			t.cardContainerLeft.appendChild(domConstruct.toDom("<br>"));

			t.billedClaimsCard = new ScoreCard({
				title: "Billed Demand Claims",
				props: {
					rowTemplate: countNetTemplate,
					getDisplayValue: getNetDisplay
				}
			});
			t.cardContainerLeft.appendChild(t.billedClaimsCard.domNode);
			t.cardContainerLeft.appendChild(domConstruct.toDom("<br>"));

			/*t.claimFilesCard = new ScoreCard({
				title: "Claim Files",
				props: {
					rowTemplate: countTemplate
				}
			});
			t.cardContainerLeft.appendChild(t.claimFilesCard.domNode);

			t.claimResponseFilesCard = new ScoreCard({
				title: "Claim Response Files",
				props: {
					rowTemplate: countTemplate
				}
			});
			t.cardContainerRight.appendChild(t.claimResponseFilesCard.domNode);*/
		},

		postCreate: function() {
			var t = this;
			t.inherited(arguments);

			t.unbilledClaimsCard.set("props",{
				onClickTitle: function() { t.handleRowClick(1,"unbilledClaims"); },
				onClickRow: function(rowId) { t.handleRowClick(1,"unbilledClaims",rowId); }
			});
			t.billedClaimsCard.set("props",{
				onClickTitle: function() { t.handleRowClick(2,"billedClaims"); },
				onClickRow: function(rowId) { t.handleRowClick(2,"billedClaims",rowId); }
			});
			/*t.claimFilesCard.set("props",{
				onClickTitle: function() { t.handleRowClick(3,"claimFiles"); },
				onClickRow: function(rowId) { t.handleRowClick(3,"claimFiles",rowId); }
			});
			t.claimResponseFilesCard.set("props",{
				onClickTitle: function() { t.handleRowClick(4,"claimResponseFiles"); },
				onClickRow: function(rowId) { t.handleRowClick(4,"claimResponseFiles",rowId); }
			});*/
		},

		refreshRendering: function() {
			var t = this;
			if (!t.data) {
				var promise = service.getDashboardData(1).then(function(data) {
					t.set("data",data);
				},utils.showMessage);
				t.addReadyPromise(promise);
				t.props.onSetLoadingPromise(promise);
			}
			t.inherited(arguments);
		},

		_setDataAttr: function(data) {
			var t = this;
			t.data = data;

			t.unbilledClaimsCard.set({ props: { data: data.unbilledClaims } } );
			t.billedClaimsCard.set({ props: { data: data.billedClaims } } );
			//t.claimFilesCard.set({ props: { data: data.claimFiles } } );
			//t.claimResponseFilesCard.set({ props: { data: data.claimResponseFiles } } );
		},

		handleRowClick: function(viewIndex,key,id) {
			var t = this;
			var filter = null;
			if (id) {
				filter = arrayUtils.filter(t.data[key],function(item) {
					return item.id === id;
				})[0].filter;
			}

			var viewMap = {
				"unbilledClaims"        : { name: "UnbilledClaims", filter: "{\"Process Date\":\"-30days\"}" },
				"billedClaims"          : { name: "BilledClaims", filter: "{\"Process Date\":\"-30days\"}" } //,
				//"claimFiles"            : { name: "ClaimFiles", filter: "{\"Generated\":\"-30days\"}" },
				//"claimResponseFiles"    : { name: "ClaimResponseFiles", filter: "{\"Received\":\"-30days\"}" }
			}
			if (filter === null) {
				filter = viewMap[key].filter;
			}

			t.props.onSetViewFilters(viewMap[key].name,json.parse(filter));
			t.props.onSwitchView(viewIndex);
		},

		refreshData: function() {
			var t = this;
			t.data = null;
			if (t.isShown) { 
				t.refreshRendering();
			}
		}
	});
});