define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/_EditorComponent",
    "thornberry/app/Component/_ViewMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "thornberry/app/service/BillingManagement", 
    "thornberry/app/service/Settings",
    "dojo/text!./Templates/CreateFileView.html",
    "thornberry/app/utils",
	"thornberry/app/Service/EnsembleFactory"
],function(
    declare,
    lang,
    arrayUtils,
    _EditorComponent,
    _ViewMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    service,
    settingsService,
    template,
    utils,
	ensembleFactory
) {
    return declare([_EditorComponent,_ViewMixin,_TemplatedMixin,_WidgetsInTemplateMixin], {
        title: "Create File",

        templateString: template,

        // claimIds: null,
        completed: false,
        settings: null,

        settingsPromise: null,

        getInitialProps: function() {
			let properties = lang.mixin(this.inherited(arguments),{
                value: {
                    subscriberDateTimePeriodDate: utils.conversions.date("",new Date(),"get")
                },
                onComplete: null
            });
			
			if (this.props.useInterface) {
				this.title = "Create Request"
			} else {
				this.title = "Create File"
			}
			
            return properties;
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                content: t.domNode,
                actions: [
                    { label: "Cancel", onClick: t.props.onCloseView },
                    { label: (t.props.useInterface ? "Create Request" : "Create File"), onClick: function() {t.createFile(t.props.useInterface); }},
                ]
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);
            t.addRequirement("subscriberDateTimePeriodDate",true);
        }, 

        refreshRendering: function() {
            var t = this;
            if (!t.settings) {
                t.addReadyPromise(settingsService.get("insuranceEligibility").then(function(settings) {
                    t.settings = settings;
                }));
            }
            t.inherited(arguments);
        },

        createFile: function(useInterface) {
            var t = this;
            if (!t.checkValid()) {
                utils.showMessage("Please correct any invalid fields.");
            } else if (!t.checkRequired()) {
                utils.showMessage("Please fill in all required fields.");
            } else {
				var value = t.get("value");
				
				var subscriberDateTimePeriodX12 = ('' + value.subscriberDateTimePeriodDate).replaceAll('-','')
				
				var service = ensembleFactory("Billing.Eligibility.EnsembleRequest");
				var promise = service({
						policyIds:t.props.selectedIds,
						useInterface:!!useInterface,
						subscriberDateTimePeriod:subscriberDateTimePeriodX12
				});
				promise.then(function(result) {
					t.loadSubView("requestSummary",result);
					t.refreshData();
				},utils.showMessage);
				t.props.onSetLoadingPromise(promise);
            }
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);
            t.addSubView("requestSummary","thornberry/app/Component/InsuranceEligibility/RequestSummaryView",function() {
                    return {
						onComplete: new function() { t.props.onCloseView(); }
					};
            });
            if (t.completed) {
                t.props.onCloseView();
            }
        },

        onBeforeClose: function() {
            var t = this;
            if (t.completed && t.props.onComplete) {
                t.props.onComplete();
                t.completed = false;
            }  
            return true;
        },

        resize: function() {
            var t = this;
            t.grid.resize();
        }
    })
});