define([
	"dojo/_base/declare",
	"thornberry/app/Component/_Component",
	"thornberry/app/Component/_EditorViewMixin",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./Templates/AddEditSequenceView.html",
	"dojo/_base/lang",
	"dojo/_base/array",
	"gridx/Grid",
	"gridx/core/model/cache/Sync",
	"gridx/modules/SingleSort",
	"dojo/store/Memory",
	"thornberry/app/Service/PatientInsurance",
	"thornberry/app/utils",
	"thornberry/app/Service/Settings",
	"thornberry/app/Component/PatientInsurance/SequenceEditor",
	"dijit/layout/ContentPane",
	"dijit/layout/LayoutContainer"
],function(
	declare,
	_Component,
	_EditorViewMixin,
	_TemplatedMixin,
	_WidgetsInTemplateMixin,
	template,
	lang,
	arrayUtils,
	Grid,
	Cache,
	SingleSort,
	Memory,
	patientInsuranceService,
	utils,
	settingsService
) {
	/*
		Author: rwitherell
		Created: *Unknown*
		History:
		2022-09-07 WRA ND-11655 Added patientType props for demand billing
	*/
	return declare([_Component,_TemplatedMixin,_WidgetsInTemplateMixin,_EditorViewMixin], {
		templateString: template,

		// properties
		title: "Sequence",
		helpPath: "/help/PatientInsurance.html#sequence-editor",
		ndocNum: "",
		sequence: null,
		sequenceId: "",
		policies: null,
		sequenceStore: null,
		billingComponentSettings: null,

		// _EditorContainerMixin props
		lockCheckEnabled: true,
		objectType: "Patient.Insurance.Sequence",

		// sub-widgets
		editor: null,

		constructor: function() {
			var t = this;
			t.sequenceStore = new Memory();
		},

		getInitialProps: function() {
			return lang.mixin(this.inherited(arguments),{
				ndocNum: "",
				objectId: ""
			});
		},

		buildRendering: function() {
			var t = this;
			t.inherited(arguments);

			t.policyGrid = new Grid({
				cacheClass: Cache,
				store: new Memory(),
				structure: [
					{ id: "insurance", field: "insurance", name: "Insurance" },
					{ id: "policyNum", field: "policyNum", name: "Policy Number" },
					{ id: "claim", field: "claim", name: "Claim" },
					{ id: "effectiveDate", field: "effectiveDate", name: "Effective Date", formatter: utils.formatters.date("effectiveDate")  }, 
					{ id: "stopDate", field: "stopDate", name: "Stop Date", formatter: utils.formatters.date("stopDate")  }, 
					{ id: "verifyDate", field: "verifyDate", name: "Verify Date", formatter: utils.formatters.date("verifyDate") }    
				],
				modules: [SingleSort],
				autoHeight: true
			});
			t.policyGridContainer.appendChild(t.policyGrid.domNode);
			t.policyGrid.startup();   
			
			t.addSubComponent(t.editor);
		},

		refreshRendering: function() {
			var t = this;
			if (t.isShown) {
				t.set("ndocNum",t.props.ndocNum);
				t.set("sequenceId",t.props.objectId);

				if (!t.billingComponentSettings) {
					t.addReadyPromise(settingsService.get("billingComponent").then(function(result) {
						t.billingComponentSettings = result;
						t.editor.set("props",{
							billingComponentSettings: t.billingComponentSettings
						});
					}));
				}

				t.editor.set("props",{
					value: t.sequence,
					mode: (t.props.objectId == "" ? "add" : "edit"),
					policies: t.policies,
					sequences: t.sequenceStore.data,
					onChangeModified: lang.hitch(t,"setPageProps"),
					disabled: (t.props.mode === "view"),
					onValidateDate: lang.hitch(t,"onValidateEffectiveDate")
				});
			}
			t.inherited(arguments);
		},

		_setNdocNumAttr: function(value) {
			var t = this;
			if (value && t.ndocNum !== value) {
				t.updatePolicies([]);
				t.updatePolicySequences([]);

				t.addReadyPromise(patientInsuranceService.getPolicies(value));
				t.addReadyPromise(patientInsuranceService.getPolicySequences(value));
				t.addReadyPromise(patientInsuranceService.getGeneral(value));

				t.setHandle("watchPolicySequences", patientInsuranceService.watchPolicySequences(value,lang.hitch(t,"updatePolicySequences")));
				t.setHandle("watchPolicies",        patientInsuranceService.watchPolicies       (value,lang.hitch(t,"updatePolicies")));
				t.setHandle("watchGeneral",			patientInsuranceService.watchGeneral		(value,lang.hitch(t,"updateGeneral")));
			}
			t.ndocNum = value;
		},

		_setSequenceIdAttr: function(value) {
			var t = this;
			t.sequenceId = value;
			t.updateSequence();
		},

		updatePolicies: function(data) {
			var t = this;
			t.policyGrid.setStore(new Memory({
				data: arrayUtils.filter(data,function(item) {return !item.isErrored;})
			}));
			t.policyGrid.model.clearCache();
			t.policyGrid.body.refresh();

			t.policies = data;
			t.editor.set("props",{
				policies: data
			});
		},

		updatePolicySequences: function(data) {
			var t = this;
			t.sequenceStore.setData(data);
			t.updateSequence();
		},

		updateGeneral: function(data) {
			var t = this;
			data = data.length ? data[0] : {};
			t.editor.set("props",{
				"patientType" : data.patientType
			});
		},

		updateSequence: function(id) {
			var t = this;
			t.sequence = t.sequenceStore.get(id || t.sequenceId) || null;
		},

		onShow: function() {
			var t = this;
			t.layoutContainer.resize();
			t.policyGrid.resize();
		},

		save: function() {
			var t = this;
			var data = lang.mixin(t.editor.get("value"),{
				ndocNum: t.props.ndocNum,
				id: t.props.objectId
			});
			return patientInsuranceService.savePolicySequence(data).then(function() {
				patientInsuranceService.getPolicySequences(t.props.ndocNum,true);
				patientInsuranceService.getPolicies(t.props.ndocNum,true);
				t.objectId = ""; // Force the editor to reload the object if the same object is edited immediately after
			});
		},

		onValidateEffectiveDate: function(date) {
			var t = this;
			return patientInsuranceService.lockSequenceDate(t.ndocNum,date).then(function(id) {
				t.unlock(t._currentLockId);
				t._currentLockId = id;
				t._lockStore.put({ id: id, acquired: true });
				return true;
			},function() {
				utils.showMessage("A sequence for that effective date is being edited in another session.");
				return false;
			});
		}
	});
});