define([
	"dojo/_base/declare",
	"thornberry/app/Component/_EditorComponent",
	"thornberry/app/Component/_TableFieldEditorMixin",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dojo/text!./Templates/SequenceEditorSingleColumn.html",
	"dojo/text!./Templates/SequenceEditorDoubleColumn.html",
	"dojo/_base/lang",
	"dojo/dom-style",
	"dojo/_base/array",
	"thornberry/app/utils",
    "thornberry/object/TableManager",
	"dijit/form/Select",
	"dijit/form/DateTextBox",
	"dijit/form/Form"
],function(
	declare,
	_EditorComponent,
	_TableFieldEditorMixin,
	_TemplatedMixin,
	_WidgetsInTemplateMixin,
	singleColumnTemplate,
	doubleColumnTemplate,
	lang,
	domStyle,
	arrayUtils,
	utils,
    TableManager
) {
	/*
		Author: rwitherell
		Created: *Unknown*
		History:
		2022-09-07 WRA ND-11655 Added demand billing
	*/
	return declare([_EditorComponent,_TemplatedMixin,_WidgetsInTemplateMixin,_TableFieldEditorMixin], {
		layout: "single-column", // single-column, or double-column
		templateString: singleColumnTemplate,

		postMixInProperties: function() {
			var t = this;
			t.inherited(arguments);
			t.templateString = (t.layout === "single-column" ? singleColumnTemplate : doubleColumnTemplate);
		},

		getInitialProps: function() {
			return lang.mixin(this.inherited(arguments),{
				value: null,
				policies: null,
				sequences: null,
				onValidateDate: null,
				billingComponentSettings: {}
			});
		},

		buildRendering: function() {
			var t = this;
			t.inherited(arguments);

			t.addValidation(["effectiveDate"],function(name,value) {
				var ok = true;
				if (t.props.onValidateDate) {
					ok = t.props.onValidateDate(utils.conversions.date(name,value,"get"));
				}
				return ok;
			});

			t.addLogic(["effectiveDate"],function(name,value) {
				t.refreshTableField("primaryPolicyId");
				t.refreshTableField("secondaryPolicyId");
				t.refreshTableField("tertiaryPolicyId");
			});
			t.addLogic(["primaryPolicyId"],function(name,value) {
				t.refreshTableField("secondaryPolicyId",(value ? undefined : ""));
				t.refreshTableField("tertiaryPolicyId",(value ? undefined : ""));
				t.refreshTableField("demandBilling",(value ? undefined : ""));
			});
			t.addLogic(["secondaryPolicyId"],function(name,value) {
				t.refreshTableField("tertiaryPolicyId",(value ? undefined : ""));
				t.refreshTableField("demandBilling",(value ? undefined : ""));
			});
			t.addLogic(["tertiaryPolicyId"],function(name,value) {
				t.refreshTableField("tertiaryPolicyId",(value ? undefined : ""));
				t.refreshTableField("demandBilling",(value ? undefined : ""));
			});
			t.addRequirement(["effectiveDate","primaryPolicyId"],true);
			t.addDisabled("secondaryPolicyId",function(name,value) {return !value.primaryPolicyId});
			t.addDisabled("tertiaryPolicyId",function(name,value) {return !value.secondaryPolicyId});
			t.addDisabled("demandBilling",function(name,value) {return !value.demandBilling});

			t.addTableFilter(["primaryPolicyId","secondaryPolicyId","tertiaryPolicyId"],function(name,item,value) {
				if (item.isErrored) { return false; }
				if (name === "secondaryPolicyId" || name === "tertiaryPolicyId") {
					var primary = t.fieldMap["primaryPolicyId"][0].get("value");
					if (!primary || primary == item.id) { return false; }
				}
				if (name === "tertiaryPolicyId") {
					var secondary = t.fieldMap["secondaryPolicyId"][0].get("value");
					if (!secondary || secondary == item.id) { return false; }
				}

				var valid = true;
				var history = t.getHistory(0);
				var effectiveDate = t.fieldMap["effectiveDate"][0].get("value");
				if (effectiveDate && effectiveDate !== history.effectiveDate) {
					var formatted = utils.conversions.date("",effectiveDate,"get");
					if ((formatted < item.effectiveDate) || (formatted >= item.stopDate && item.stopDate)) {
						valid = false;
					}
				}
				return valid;
			});


			t.addTableFilter("demandBilling",function(name,item,value) {
				var secondary = t.fieldMap["secondaryPolicyId"][0].get("value");
				var tertiary = t.fieldMap["tertiaryPolicyId"][0].get("value");

				var getBillingType = function(policyId) {
					var billingType;
					arrayUtils.some(t.props.policies, function(value) {
						if (value.id == policyId) {
							var insTableObj = t.tableManager.get("Tables.Insurance").get(value.insTabId);
							if (insTableObj) {
								billingType = insTableObj.BillingType;
							}
							return true;
						}
					});
					return billingType;
				};

				var valid = true;
				if (item.active == "No") {
					valid = false;
				} else if (item.id == 2) {
					valid = (secondary != "") && (getBillingType(secondary) == 1);
				} else if (item.id == 3) {
					valid = (tertiary != "") && (getBillingType(tertiary) == 1);
				}

				return valid;
			});
		},

		refreshRendering: function() {
			var t = this;
			
			var policies = [];
			if (t.props.policies) {
				policies = arrayUtils.map(t.props.policies,function(policy) {
					return { id: policy.id, name: policy.insurance+(policy.policyNum !== "" ? " (Policy #"+policy.policyNum+")" : ""), effectiveDate: policy.effectiveDate, stopDate: policy.stopDate, isErrored: policy.isErrored }
				});
			}
			t.setTable("policy",policies);

			if (t.props.billingComponentSettings && t.props.billingComponentSettings.enabledNow && t.props.patientType == "homecare") {
				domStyle.set(t.demandBillingContainer,"display","");
			} else {
				domStyle.set(t.demandBillingContainer,"display","none");
			}

			t.inherited(arguments);   
		},

		_needsRefresh: function(value) {
			return true;
		}
	});
});