/*
    Author: Will Amesbury
    Created: 2022-09-22
    History:

    Notes: Uses much of the Billing Management infrastructure
*/
define([
	"dojo/_base/declare",
	"thornberry/app/_Module",
	"thornberry/app/Component/LazyLoadView",
	"thornberry/app/Component/DemandBills/Dashboard",
	"thornberry/app/Component/BillingManagement/UnbilledDemandClaimsView",
	"thornberry/app/Component/BillingManagement/BilledDemandClaimsView",
	//"thornberry/app/Component/BillingManagement/ClaimFilesView",
	//"thornberry/app/Component/BillingManagement/ClaimResponseFilesView",
	"thornberry/app/Component/BillingManagement/ClaimView",
	"thornberry/app/Service/BillingManagement",
	"dojo/dom-attr",
	"dojo/aspect",
	"thornberry/app/utils"
],function(
	declare,
	_Module,
	LazyLoadView,
	Dashboard,
	UnbilledClaimsView,
	BilledClaimsView,
	//ClaimFilesView,
	//ClaimResponseFilesView,
	ClaimView,
	service,
	domAttr,
	aspect,
	utils
) {
	onOpenClaim = null; // global method for use in 

	return declare(_Module,{
		title: "Demand Bills",

		initPage: function() {
			var t = this;
			var page = t.inherited(arguments);

			onOpenClaim = function(id) {
				var view = new ClaimView({
					props: {
						viewOnly: true,
						mode: "view",
						objectId: id
					}
				});
				page.pushViewContainers(view);
			};

			var onSearchClaim = function(claimId) {
				page.setLoadingPromise(
					service.findClaimId(claimId).then(onOpenClaim,utils.showMessage)
				);
			}

			aspect.before(page,"refreshRendering",function() {
				page.props.searchButtonShown = true;
				page.props.onSearch = onSearchClaim;
			});
			page.searchInput.set("placeHolder","Enter Claim ID");
			domAttr.set(page.searchButtonNode,"title","Find Claim by ID");

			return page;
		},

		initViews: function(page) {
			var viewMap = {}
			var props = {
				props: {
					onSetViewFilters: function(viewName,filters) {
						var view = viewMap[viewName];
						if (view) {
							view.load().set({
								props: {
									filters: filters
								}
							})
						}
					},
					onUpdateData: function() {
						for (var viewName in viewMap) {
							viewMap[viewName].instance.refreshData();
						}
					}
				},
				demand: true
			};

			viewMap["Dashboard"]            = new LazyLoadView({ viewClass: Dashboard,              title: "Dashboard",             props: props });
			viewMap["UnbilledClaims"]       = new LazyLoadView({ viewClass: UnbilledClaimsView,     title: "Unbilled Claims",       props: props,   preload: true });
			viewMap["BilledClaims"]         = new LazyLoadView({ viewClass: BilledClaimsView,       title: "Billed Claims",         props: props,   preload: true });
			//viewMap["ClaimFiles"]           = new LazyLoadView({ viewClass: ClaimFilesView,         title: "Claim Files",           props: props,   preload: true });
			//viewMap["ClaimResponseFiles"]   = new LazyLoadView({ viewClass: ClaimResponseFilesView, title: "Claim Response Files",  props: props,   preload: true });

			page.pushViewContainers([
				viewMap["Dashboard"],
				viewMap["UnbilledClaims"],
				viewMap["BilledClaims"] //,
				//viewMap["ClaimFiles"],
				//viewMap["ClaimResponseFiles"]
			]);
		}
	});
});