// thornberry/patientStatus/LevelOfCareWidget.js
define([
	"dojo/_base/declare",
	"thornberry/object/ObjectEditor",
	"thornberry/object/ObjectModel",
	"thornberry/object/TableManager",
	"thornberry/serverRequest",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/aspect",
	"dojo/date/locale",
	"dojo/dom-style",
	"dojo/dom-class",
	"dojo/on",
	"dijit/registry",
	"dijit/form/Button",
	"gridx/Grid",
	"gridx/core/model/cache/Sync",
	"gridx/modules/IndirectSelect",
	"gridx/modules/select/Row",
	"gridx/modules/RowHeader",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin"
], function(declare, ObjectEditor, ObjectModel, TableManager, serverRequest, arrayUtils, lang, aspect, locale, domStyle, domClass, on, registry,
			Button, Grid, Cache, IndirectSelect, SelectRow, RowHeader, _WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin) {
				
	var fieldProperties = [
		{
			name: "StartDate",
			displayName: "Start Date",
			type: "date",
			required: true
		},
		{
			name: "LevelOfCare",
			displayName: "Level of Care",
			type: "tableItem",
			tableName: "levelOfCareStore",
			required: true
		},
		{
			name: "HospiceLocation",
			displayName: "Hospice Location",
			type: "tableItem",
			tableName: "Tables.PatientStatusLocation",
			required: true
		},
		{
			name: "HospiceFacility",
			displayName: "Hospice Facility",
			type: "tableItem",
			tableName: "Tables.OutsideTables.Facility",
			required: true
		},
		{
			name: "CBSA",
			displayName: "CBSA",
			type: "text"
		},
		{
			name: "RUGScore",
			displayName: "RUG Score",
			type: "tableItem",
			tableName: "Tables.RugScore"
		}
	];

	var fieldModel = new ObjectModel({ propertyDefinitions: fieldProperties });
	var LevelOfCare = fieldModel.getObject();

	var fieldTemplate = '<div>'+
							'<div class="content-flex-container">'+
								'<div class="flex-field-container" data-dojo-attach-point="StartDateContainer">'+
									'<div class="field-name field-name-large" data-dojo-attach-point="StartDateLabelContainer">'+
										'<label>Start Date</label>'+
									'</div>'+
									'<div class="field-value field-value-medium" data-dojo-attach-point="StartDateFieldContainer">'+
										'<input data-dojo-attach-point="StartDateNode">'+
									'</div>'+
								'</div>'+
							'</div>'+
							'<div class="content-flex-container">'+
								'<div class="flex-field-container" data-dojo-attach-point="LevelOfCareContainer">'+
									'<div class="field-name field-name-large" data-dojo-attach-point="LevelOfCareLabelContainer">'+
										'<label>Level of Care</label>'+
									'</div>'+
									'<div class="field-value field-value-medium" data-dojo-attach-point="LevelOfCareFieldContainer">'+
										'<input data-dojo-attach-point="LevelOfCareNode">'+
									'</div>'+
								'</div>'+
							'</div>'+
							'<div class="content-flex-container">'+
								'<div class="flex-field-container" data-dojo-attach-point="HospiceLocationContainer">'+
									'<div class="field-name field-name-large" data-dojo-attach-point="HospiceLocationLabelContainer">'+
										'<label>Patient Status Location</label>'+
									'</div>'+
									'<div class="field-value field-value-medium" data-dojo-attach-point="HospiceLocationFieldContainer">'+
										'<input data-dojo-attach-point="HospiceLocationNode">'+
									'</div>'+
								'</div>'+
							'</div>'+
							'<div class="content-flex-container">'+
								'<div class="flex-field-container" data-dojo-attach-point="HospiceFacilityContainer">'+
									'<div class="field-name field-name-large" data-dojo-attach-point="HospiceFacilityLabelContainer">'+
										'<label>Hospice Facility</label>'+
									'</div>'+
									'<div class="field-value field-value-medium" data-dojo-attach-point="HospiceFacilityFieldContainer">'+
										'<input data-dojo-attach-point="HospiceFacilityNode">'+
									'</div>'+
								'</div>'+
							'</div>'+
							'<div class="content-flex-container">' +
								'<div class="flex-field-container" data-dojo-attach-point="CBSAContainer">' +
									'<div class="field-name field-name-large" data-dojo-attach-point="CBSALabelContainer">' +
										'<label>CBSA</label>' +
									'</div>' +
									'<div class="field-value field-value-small" data-dojo-attach-point="CBSAFieldContainer">' +
										'<input data-dojo-attach-point="CBSANode">' +
									'</div>' +
								'</div>' +
							'</div>' +
							(
								(billingComponentSettings.rugScoreEnabled == 1)
								? (
									'<div class="content-flex-container">'+
										'<div class="flex-field-container" data-dojo-attach-point="RUGScoreContainer">'+
											'<div class="field-name field-name-large" data-dojo-attach-point="RUGScoreLabelContainer">'+
												'<label>RUG Score</label>'+
											'</div>'+
											'<div class="field-value field-value-medium" data-dojo-attach-point="RUGScoreFieldContainer">'+
												'<input data-dojo-attach-point="RUGScoreNode">'+
											'</div>'+
										'</div>'+
									'</div>')
								: ''
							) +
						'</div>';

	var LocFields = declare([ObjectEditor], {
		objectModel: fieldModel,
		templateString: fieldTemplate,

		postMixInProperties: function() {
			var t = this;
			t.inherited("postMixInProperties", arguments);
		},

		buildRendering: function() {
			var t = this;
			t.inherited("buildRendering", arguments);
		},

		postCreate: function() {
			var t = this;
			t.inherited(arguments);

			t.own(aspect.after(t, "_setObjectAttr", function(object) {
				var restrictIdArray = [];
				TableManager.get("Tables.PatientStatusLocation").query({HasSiteOfService: "Yes"}).forEach(function(location) {
					restrictIdArray.push(location.id);
				});
				t.populateSelect(t.HospiceLocationField, TableManager.get("Tables.PatientStatusLocation"), t.object.get("HospiceLocation"), true, restrictIdArray);
				updateCbsa();
			}, true));

			t.registerChangeLogic("HospiceLocation", function(value) {
				if (!t.inObjectLoad) {
					t.object.set("HospiceFacility", "");
				}
				var restrictIdArray = [];
				TableManager.get("Tables.OutsideTables.Facility").query().forEach(function(facility) {
					if (facility.PatientStatusLocation == value || facility.PatientStatusLocation === "" || facility.PatientStatusLocation === 0) {
						restrictIdArray.push(facility.id);
					}
				});
				t.populateSelect(t.HospiceFacilityField, TableManager.get("Tables.OutsideTables.Facility"), t.object.get("HospiceFacility"), true, restrictIdArray);
			});

			var updateCbsa = function() {
				var hospiceFacility = t.object.get("HospiceFacility");
				var hospiceLocation = t.object.get("HospiceLocation");
				var startDate = t.object.get("StartDate");

				if (hospiceLocation && TableManager.get("Tables.PatientStatusLocation").get(hospiceLocation).SiteOfService == 1) {
					t.object.set("CBSA", "See Location of Care");
				} else if (hospiceFacility && hospiceFacility != "") {
					t.getCbsa(hospiceFacility, hospiceLocation, startDate);
				} else {
					t.object.set("CBSA", "");
				}
			};

			if (t.HospiceFacilityField && t.CBSAField) {
				t.registerChangeLogic("HospiceFacility", function() {
					updateCbsa();
				});

				t.registerChangeLogic("StartDate", function () {
					updateCbsa();
				});

				t.registerChangeLogic("HospiceLocation", function () {
					updateCbsa();
				});
			}
		},

		_getValueAttr: function() {
			// summary:
			//		Support get('value')

			var t = this;
			var value = {};
			var changed = true;
			var levelOfCareSaveString = "";
			var locWidget = registry.byId(t.locWidgetId);
			var action = locWidget.action;

			if (t.object && action !== "") {
				value = t.object.getData();

				value.action = action;

				if (action === "Edit" || action === "Delete") {
					value.id = locWidget.grid.select.row.getSelected()[0];
				}

				if (action === "Add" || action === "Edit") {
					// Check for changes
					if (action === "Edit") {
						var oldObj = TableManager.get("patientLOCStore").get(value.id);
						if (value.StartDate === oldObj.StartDate &&
							value.LevelOfCare === oldObj.LevelOfCare.toString() &&
							value.HospiceLocation === oldObj.HospiceLocation.toString() &&
							value.HospiceFacility === oldObj.HospiceFacility.toString() &&
							value.RUGScore === oldObj.RUGScore.toString()) {
							changed = false;
						}
					}
				}

				if (changed) {
					levelOfCareSaveString = JSON.stringify(value);
				}
			}

			return levelOfCareSaveString;
		},

		_setDisabledAttr: function(disable) {
			var t = this;
			if (typeof disable === "undefined") {
				disable = true;
			}

			t.disable("StartDate", disable);
			t.disable("LevelOfCare", disable);
			t.disable("HospiceLocation", disable);
			t.disable("HospiceFacility", disable);
			t.disable("CBSA", true);
			t.disable("RUGScore", disable);
			if (disable && t.object) {
				t.object.set("StartDate", "");
				t.object.set("LevelOfCare", "");
				t.object.set("HospiceLocation", "");
				t.object.set("HospiceFacility", "");
				t.object.set("CBSA", "");
				t.object.set("RUGScore", "");
			}

			t.updateFieldPinking();
		},

		validate: function() {
			var t = this;
			var valid = true;
			for (var property in t.objectModel.propertyDefinitions) {
				var fieldName = t.objectModel.propertyDefinitions[property].name;

				var widget = t[fieldName + "Field"];
				if (widget && typeof widget.validate !== "undefined") {
					valid = valid && widget.validate();
				}
			}
			return valid;
		},
		
		getCbsa: function (hospiceFacility, hospiceLocation, startDate) {
			var t = this;
	
			if (startDate == null) {
				startDate = new Date();
			}
	
			return serverRequest("Patient.LevelOfCare.LevelOfCare", "GetCbsaWeb", {
				facilityId: hospiceFacility, locationId: hospiceLocation, date: startDate
			}).then(function(result) {
				t.object.set("CBSA", result);
			}, function(error) { console.error("getCbsa: " + error); });
		}
	});

	var LevelOfCareWidget = declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
		templateString: '<div class="content-area" data-dojo-attach-point="levelOfCareContainer">'+
							'<div class="content-area-title">Level of Care</div>'+
							'<div class="content-flex-container" data-dojo-attach-point="actions">'+
								'<button data-dojo-type="dijit/form/Button" type="button" data-dojo-attach-point="btnAdd" disabled>Add</button>'+
								'<button data-dojo-type="dijit/form/Button" type="button" data-dojo-attach-point="btnEdit" disabled>Edit</button>'+
								'<button data-dojo-type="dijit/form/Button" type="button" data-dojo-attach-point="btnDelete" disabled>Delete</button>'+
								'<div data-dojo-attach-point="locDeleteMessage" class="label" style="width:540px;display:none">The selected level of care has been marked for deletion. Click "Save" to complete. Click "Cancel" to undo.</div>'+
							'</div>'+
							'<div class="content-flex-container" data-dojo-attach-point="gridContainer" style="margin-bottom: 5px;"></div>'+
							'<div class="content-flex-container" data-dojo-attach-point="fieldsContainer"></div>'+
						'</div>',
		grid: {},
		fields: null,
		action: "",

		postMixInProperties: function() {
			var t = this;
			t.inherited("postMixInProperties", arguments);
		},

		buildRendering: function() {
			var t = this;
			t.inherited("buildRendering", arguments);

			var getFromStore = function(store, id, field) {
				var val = "";
				var item = TableManager.get(store).get(id);
				if (item) {
					if (item[field]) {
						val = item[field];
					}
				}
				return escapeText(val);
			};

			var dateFormatter = function(value) {
				value = value[this.field];
				return (value ? locale.format(locale.parse(value,{selector:"date",datePattern:"yyyy-MM-dd"}),{selector:"date",datePattern:"MM/dd/yyyy"}) : "");
			};

			var layout = [
				{ id: "row", field: "row", name: "Seq#", width: "40px" },
				{ id: "levelOfCare", field: "LevelOfCare", name: "Level", width: "175px", formatter: function(itemData) { return getFromStore("levelOfCareStore", itemData.LevelOfCare, "name"); } },
				{ id: "startDate", field: "StartDate", name: "Start Date", width: "80px", formatter: dateFormatter },
				{ id: "stopDate", field: "StopDate", name: "End Date", width: "80px", formatter: dateFormatter },
				{ id: "days", field: "Days", name: "Days", width: "60px" },
				{ id: "location", field: "HospiceLocation", name: "Hospice Location", width: "175px", formatter: function(itemData) { return getFromStore("Tables.PatientStatusLocation", itemData.HospiceLocation, "name"); } },
				{ id: "facility", field: "HospiceFacility", name: "Hospice Facility", width: "175px", formatter: function(itemData) { return getFromStore("Tables.OutsideTables.Facility", itemData.HospiceFacility, "name"); } },
				{ id: "cbsa", field: "CBSA", name: "CBSA", width: "60px" }
			];

			if (billingComponentSettings.rugScoreEnabled == 1) {
				layout.push({ id: "rugScore", field: "RUGScore", name: "RUG Score", width: "60px", formatter: function(itemData) { return getFromStore("Tables.RugScore", itemData.RUGScore, "name"); } });
			}

			var patientLOCStore = TableManager.get("patientLOCStore");
			
			if (!(t.grid && t.grid.domNode)) {
				var grid = new Grid({
					cacheClass: Cache,
					store: patientLOCStore,
					structure: layout,
					autoHeight: true,
					selectRowMultiple: false,
					selectRowTriggerOnCell: true,
					modules: [
						SelectRow,
						RowHeader,
						IndirectSelect
					]
				});
				
				domStyle.set(grid.domNode, "width", "882px");
				t.gridContainer.appendChild(grid.domNode);
				grid.startup();
				t.grid = grid;
			}

			if (!t.fields) {
				t.fields = new LocFields({locWidgetId: t.id});
				t.fieldsContainer.appendChild(t.fields.domNode);
			}
			t.fields.set("object", new LevelOfCare());

			t.grid.connect(t.grid.select.row, "onSelected", lang.hitch(t, t.updateActions));
			t.grid.connect(t.grid, "onCellDblClick", lang.hitch(t, t.actionEdit));
			
			t.own(aspect.after(registry.byId(t.statusEditorId), "_setObjectAttr", lang.hitch(t, function() {
				t.updateLevelStartDateConstraints();

				t.grid.connect(t.grid.body, "onAfterRow", lang.hitch(t, function(row) {
					var currentSOCDate = registry.byId(t.statusEditorId).object.get("agencySocDate");
					if (currentSOCDate) {
						var item = row.grid.model.byId(row.id).item;
						var locDate = locale.parse(item.StartDate, { selector: "date", datePattern: "yyyy-MM-dd" });
						if (locDate < currentSOCDate) {
							domClass.add(row.node(), "invalidRow");
						}
					}
				}));
			})));
		},

		postCreate: function() {
			var t = this;
			t.inherited("postCreate", arguments);
			var patientLOCStore = TableManager.get("patientLOCStore");
			
			// var levelStartDate = t.fields.StartDateField;
			// var oldRangeCheck = levelStartDate.rangeCheck;
			// levelStartDate.rangeCheck = function(date, constraints) {
				// var selectedId = t.grid.select.row.getSelected()[0];
				// for (var i = 0; i < patientLOCStore.data.length; i++) {
					// var formattedDate = locale.format(date, { selector: "date", datePattern: "MM/dd/yyyy" });
					// if (formattedDate === patientLOCStore.data[i].StartDate && selectedId !== patientLOCStore.data[i].id) {
						// return false;
					// }
				// }
				// var func = lang.hitch(levelStartDate, oldRangeCheck);
				// return func(date, constraints);
			// };

			t.set("disabled", true);
			t.fields.set("disabled", true);

			if (registry.byId(t.statusEditorId).actionType === "View") {
				domStyle.set(t.actions, "display", "none");
				domStyle.set(t.fieldsContainer, "display", "none");
			}

			t.own(on(t.btnAdd, "click", lang.hitch(t, t.actionAdd)),
				  on(t.btnEdit, "click", lang.hitch(t, t.actionEdit)),
				  on(t.btnDelete, "click", lang.hitch(t, t.actionDelete)));

			t.grid.model.clearCache();
			t.grid.body.refresh();
			t.grid.resize();
		},

		_setDisabledAttr: function(disable) {
			var t = this;
			if (typeof disable === "undefined") {
				disable = true;
			}

			if (disable) {
				var patientLOCStore = TableManager.get("patientLOCStore");
				for (var i = 0; i < patientLOCStore.data.length; i++) {
					t.grid.select.row.unselectable[patientLOCStore.data[i].id] = 1;
				}
			} else {
				t.grid.select.row.unselectable = {};
			}
			t.btnAdd.set("disabled", disable);
			if (disable) {
				t.btnEdit.set("disabled", disable);
				t.btnDelete.set("disabled", disable);
			}
		},

		updateActions: function() {
			var t = this;
			var selection = t.grid.select.row.getSelected();
			if (t.action === "" && selection.length > 0) {
				t.btnEdit.set("disabled",false);
				t.btnDelete.set("disabled",false);
			} else {
				t.btnEdit.set("disabled",true);
				t.btnDelete.set("disabled",true);
			}
		},

		updateLevelStartDateConstraints: function() {
			var t = this;
			var statusEditor = registry.byId(t.statusEditorId);
			if (statusEditor.object) {
				var dischargeDate = statusEditor.object.get("dischargeDate");
				var snaDate = statusEditor.object.get("snaDate");
				var agencySocDate = statusEditor.object.get("agencySocDate");
				var socDate = locale.parse(patientInfo.socDate, { selector: "date", datePattern: "yyyy-MM-dd" });

				var max = null;
				if (!patientInfo.trainingPatient) {
					max = new Date();
				}
				var min = null;
				var date = null;
				if (patientInfo.patientStatus === 7 && dischargeDate !== "") {
					date = dischargeDate;
					if (date < max) {
						max = date;
					}
				} else if (patientInfo.patientStatus === 5 && snaDate !== "") {
					date = snaDate;
					if (date < max) {
						max = date;
					}
				}

				if (agencySocDate || socDate !== "") {
					date = socDate;
					if (agencySocDate) {
						min = agencySocDate;
					} else if (date) {
						min = date;
					}
				}

				var levelStartDate = t.fields.StartDateField;

				var constraints = {};
				if (max) {
					constraints.max = max;
				}
				if (min) {
					constraints.min = min;
				}
				levelStartDate.set("constraints", constraints);
			}
		},

		validateLevelsOfCare: function() {
			var t = this;
			var statusEditor = registry.byId(t.statusEditorId);
			if (statusEditor.object) {
				var currentSOCDate = statusEditor.object.get("agencySocDate");
				if (currentSOCDate) {
					var patientLOCStore = TableManager.get("patientLOCStore");
					for (var i = 0; i < patientLOCStore.data.length; i++) {
						var locDate = locale.parse(patientLOCStore.data[i].StartDate, { selector: "date", datePattern: "yyyy-MM-dd" });
						if (locDate < currentSOCDate) {
							actionShowMessage("This patient's levels of care are prior to this start of care. Please review and correct any discrepancies.");
							break;
						}
					}
				}

				t.grid.model.clearCache();
				t.grid.body.refresh();
			}
		},

		validate: function() {
			return this.fields.validate();
		},

		actionAdd: function() {
			var t = this;
			t.action = "Add";

			t.set("disabled", true);
			t.fields.set("disabled", false);
			t.grid.select.row.clear();

			t.fields.set("object", new LevelOfCare());
		},

		actionEdit: function() {
			var t = this;
			var selectedId = t.grid.select.row.getSelected()[0];
			if (selectedId && t.action === "") {
				t.action = "Edit";
				t.set("disabled", true);
				t.fields.set("disabled", false);

				var selectedLOC = TableManager.get("patientLOCStore").get(selectedId);
				t.fields.set("object", new LevelOfCare(selectedLOC));
			}
		},
		
		actionDelete: function() {
			var t = this;
			if (patientInfo.patientStatus === 6 && TableManager.get("patientLOCStore").data.length === 1) {
				actionShowMessage("You must have at least one level of care for an admitted patient.");
			} else {
				t.action = "Delete";
				t.set("disabled",true);
				domStyle.set(t.locDeleteMessage, "display", "inline");
			}
		},

		_getValueAttr: function() {
			// summary:
			//		Support get('value')

			var t = this;
			var value = "";

			if (t.fields && t.action !== "") {
				value = t.fields.get("value");
			}

			return value;
		}
	});

	return LevelOfCareWidget;
}
);